/*
 * This file is part of Spout.
 *
 * Copyright (c) 2011 Spout LLC <http://www.spout.org/>
 * Spout is licensed under the Spout License Version 1.
 *
 * Spout is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * In addition, 180 days after any changes are published, you can use the
 * software, incorporating those changes, under the terms of the MIT license,
 * as described in the Spout License Version 1.
 *
 * Spout is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License,
 * the MIT license and the Spout License Version 1 along with this program.
 * If not, see <http://www.gnu.org/licenses/> for the GNU Lesser General Public
 * License and see <http://spout.in/licensev1> for the full license, including
 * the MIT license.
 */
package org.spout.api.command.annotated;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.spout.api.entity.state.PlayerInputState.MouseDirection;
import org.spout.api.input.Keyboard;

/**
 * Represents a key-binding used with the {@link CommandDescription} annotation in {@link AnnotatedCommandExecutor}s.
 * If there is not also a {@link Platforms} annotation, this will only be a local binding.
 */
@Target (ElementType.METHOD)
@Retention (RetentionPolicy.RUNTIME)
public @interface Binding {
	/**
	 * Returns the keys that this command is bound to.
	 *
	 * @return keys command is bound to
	 */
	public Keyboard[] value() default {};

	/**
	 * Returns the mouse buttons this command is bound to.
	 *
	 * @return mouse buttons
	 */
	public int[] mouse() default {};

	/**
	 * Returns the mouse directions this command is bound to.
	 *
	 * @return mouse buttons
	 */
	public MouseDirection[] mouseDirections() default {};

	/**
	 * Returns true if this binding should be run asynchronously.
	 *
	 * @return true if should run async
	 */
	public boolean async() default false;
}
