/*
 * This file is part of Spout.
 *
 * Copyright (c) 2011 Spout LLC <http://www.spout.org/>
 * Spout is licensed under the Spout License Version 1.
 *
 * Spout is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * In addition, 180 days after any changes are published, you can use the
 * software, incorporating those changes, under the terms of the MIT license,
 * as described in the Spout License Version 1.
 *
 * Spout is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License,
 * the MIT license and the Spout License Version 1 along with this program.
 * If not, see <http://www.gnu.org/licenses/> for the GNU Lesser General Public
 * License and see <http://spout.in/licensev1> for the full license, including
 * the MIT license.
 */
package org.spout.api.event.server;

import org.spout.api.data.DataSubject;
import org.spout.api.data.DataValue;
import org.spout.api.data.ValueHolder;
import org.spout.api.event.HandlerList;
import org.spout.api.geo.World;

/**
 * This event is called when {@link DataSubject#getData(String)} is called.
 */
public class RetrieveDataEvent extends NodeBasedEvent {
	private final DataSubject subject;
	private final World world;
	private ValueHolder result;
	private static final HandlerList handlers = new HandlerList();

	public RetrieveDataEvent(World world, DataSubject subject, String node) {
		super(node);
		this.world = world;
		this.subject = subject;
	}

	public World getWorld() {
		return world;
	}

	/**
	 * Gets the subject the data is being taken from.
	 *
	 * @return subject of data
	 */
	public DataSubject getSubject() {
		return subject;
	}

	/**
	 * Returns the raw result of the event.
	 *
	 * @return object
	 */
	public ValueHolder getResult() {
		return result;
	}

	/**
	 * Sets the result of the event.
	 */
	public void setResult(ValueHolder result) {
		this.result = result;
	}

	/**
	 * Sets the result of the event.
	 */
	public void setResult(Object result) {
		setResult(new DataValue(result));
	}

	@Override
	public HandlerList getHandlers() {
		return handlers;
	}

	public static HandlerList getHandlerList() {
		return handlers;
	}
}
